import os,sys
from gladevcp.persistence import IniFile,widget_defaults,set_debug,select_widgets
import hal
import hal_glib
import gtk
import glib
import linuxcnc
import time

debug = 1
c = linuxcnc.command()
s = linuxcnc.stat()
config_ini_file = linuxcnc.ini(os.getenv("INI_FILE_NAME"))
axes = int(config_ini_file.find("TRAJ", "AXES") or 0)

def all_axes_homed():
    print "axes: %i" % axes
    axes_home= s.homed 
    print axes_home
    print axes_home.count(1)
    if axes_home.count(1) == axes:
        print ("All axes homed")
        return 1
    else:    
        print ("At least one axes is not homed")
        return 0

def ok_for_mdi():
    s.poll()
    print ("OK for MDI Mode")
    print ("estop: %i" % s.estop)
    print ("enabled: %i" % s.enabled)
    print ("interp_state: %i" % s.interp_state)
    return not s.estop and s.enabled and all_axes_homed() and (s.interp_state == linuxcnc.INTERP_IDLE)

def ok_for_manual():
    s.poll()
    print ("OK for Manual Mode")
    print ("estop: %i" % s.estop)
    print ("enabled: %i" % s.enabled)
    print ("interp_state: %i" % s.interp_state)
    print ("task_mode: %i" % s.task_mode)
    return not s.estop and s.enabled and (s.interp_state == linuxcnc.INTERP_IDLE) and (s.task_mode <> linuxcnc.MODE_AUTO)


class HandlerClass:
    '''
    class with gladevcp callback handlers
    '''
    S1 = 0
    S2 = 0
    S3 = 0        
    def on_pulley_toggled(self,widget,data=None):
        
        if widget.get_active():         
            self.builder.get_object('spindle-rpm-hbar').max = float(self.max_speed_high_gear)
        else:
            self.builder.get_object('spindle-rpm-hbar').max = float(self.max_speed_low_gear)
        self.builder.get_object('spindle-rpm-hbar').queue_draw()
        self.spindle_rpm_hbar_max = self.builder.get_object('spindle-rpm-hbar').max
        print "Toggle Pulley"        

    def on_stop_clicked(self,widget,data=None):
        c.spindle(0)
        print "Stop Clicked"
        
    def on_s1_clicked(self,widget,data=None):
        if ok_for_mdi():
            c.mode(linuxcnc.MODE_MDI)
            c.wait_complete() # wait until mode switch executed
            c.mdi("M3 S%d" % self.S1)
            print "S1: M3 S%d" % self.S1

    def on_s2_clicked(self,widget,data=None):
        if ok_for_mdi():
            c.mode(linuxcnc.MODE_MDI)
            c.wait_complete() # wait until mode switch executed
            c.mdi("M3 S%d" % self.S2)
            print "S2: M3 S%d" % self.S2

    def on_s3_clicked(self,widget,data=None):
        if ok_for_mdi():
            c.mode(linuxcnc.MODE_MDI)
            c.wait_complete() # wait until mode switch executed
            c.mdi("M3 S%d" % self.S3)
            print "S3: M3 S%d" % self.S3
    
    def on_destroy(self,obj,data=None):
        self.ini.save_state(self)
 
    def __init__(self, halcomp,builder,useropts):
        '''
        Handler classes are instantiated in the following state:
        - the widget tree is created, but not yet realized (no toplevel window.show() executed yet)
        - the halcomp HAL component is set up and the widhget tree's HAL pins have already been added to it
        - it is safe to add more hal pins because halcomp.ready() has not yet been called at this point.

        after all handlers are instantiated in command line and get_handlers() order, callbacks will be
        connected with connect_signals()/signal_autoconnect()

        The builder may be either of libglade or GtkBuilder type depending on the glade file format.
        '''

        self.halcomp = halcomp
        self.builder = builder
        self.useropts = useropts

#        config_ini_file = linuxcnc.ini(os.getenv("INI_FILE_NAME"))
        self.max_speed_high_gear = float(config_ini_file.find("SPINDLE", "MAX_RPM_HIGH") or 0.0)
        self.max_speed_low_gear = float(config_ini_file.find("SPINDLE", "MAX_RPM_LOW") or 0.0)
        self.S1 = float(config_ini_file.find("SPINDLE", "S1") or 100.0)
        self.S2 = float(config_ini_file.find("SPINDLE", "S2") or 1000.0)
        self.S3 = float(config_ini_file.find("SPINDLE", "S3") or 2800.0)
        spindle_S1=self.builder.get_object('hal_spindle_s1')
        spindle_S2=self.builder.get_object('hal_spindle_s2')
        spindle_S3=self.builder.get_object('hal_spindle_s3')
        spindle_S1.set_label("%d RPM" % self.S1)
        spindle_S2.set_label("%d RPM" % self.S2)
        spindle_S3.set_label("%d RPM" % self.S3)


        print "High Gear: %d" % self.max_speed_high_gear
        print "Low Gear: %d" % self.max_speed_low_gear
  
        self.defaults = {
            IniFile.vars : { 'spindle_rpm_hbar_max' : self.max_speed_high_gear},
            IniFile.widgets: widget_defaults(select_widgets(self.builder.get_objects(), hal_only=True,output_only = True))
        }
        self.ini_filename = __name__ + ".ini"
        print "Ini File Name: %s" % self.ini_filename        
        self.ini = IniFile(self.ini_filename,self.defaults,self.builder)
        self.ini.restore_state(self)
        self.builder.get_object('spindle-rpm-hbar').max= float(self.spindle_rpm_hbar_max)
        self.builder.get_object('spindle-rpm-hbar').queue_draw()
 
def get_handlers(halcomp,builder,useropts):
    '''
    this function is called by gladevcp at import time (when this module is passed with '-u <modname>.py')

    return a list of object instances whose methods should be connected as callback handlers
    any method whose name does not begin with an underscore ('_') is a  callback candidate

    the 'get_handlers' name is reserved - gladevcp expects it, so do not change
    '''
    return [HandlerClass(halcomp,builder,useropts)]
